# -*- encoding: utf-8 -*-

from django import forms
from django.forms.widgets import Select
from models import NOM_CHAMPS_ETAT, TEXTE_CHAMPS_ETAT, HELP_CHAMPS_ETAT, WorkflowCommentaire

def form_initialiser_etats(form, request, instance=None):
    """
    Modifier le champs 'etat' à travers le form, pour lui fournir la liste
    des états en fonction du contexte.
    """
    # ModelForm
    fields = getattr(form, 'fields', None)
    # AdminForm
    if not fields:
        fields = getattr(form, 'base_fields', None)
    if instance:
        choices = instance.get_etats_disponibles(request)
        fields[NOM_CHAMPS_ETAT].choices = choices
        fields[NOM_CHAMPS_ETAT].widget = Select(choices=choices)
    else:
        etat_initial = form.Meta.model.etat_initial
        libelle_etat_initial = form.Meta.model.etats[etat_initial]
        fields[NOM_CHAMPS_ETAT].initial = etat_initial
        choices = [(etat_initial, libelle_etat_initial), ]

        fields[NOM_CHAMPS_ETAT].choices = choices
        fields[NOM_CHAMPS_ETAT].widget = Select(choices=choices)


class WorkflowFormMixin(forms.ModelForm):
    """
    Mixin pour crééer un formulaire avec l'état dans le frontend.
    Pour faire un formulaire dans l'application, spécifier le modèle ainsi que les
    champs à afficher.
    """
    etat = forms.ChoiceField(label=TEXTE_CHAMPS_ETAT, help_text=HELP_CHAMPS_ETAT)
    commentaire = forms.CharField(widget=forms.Textarea, required=False)

    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request', None)
        if self.request is None:
            raise Exception("Workflow form doit est construit en connaissant le request")
        instance = kwargs.get('instance', None)
        super(WorkflowFormMixin, self).__init__(*args, **kwargs)
        form_initialiser_etats(self, request=self.request, instance=instance)

    def save(self, *args, **kwargs):
       commentaire = WorkflowCommentaire()
       commentaire.content_object = self.instance
       commentaire.texte = self.data.get('commentaire', '')
       commentaire.etat_initial = self.instance._etat_courant
       commentaire.etat_final = self.instance.etat
       commentaire.owner = self.request.user
       commentaire.save()
       return super(forms.ModelForm, self).save(*args, **kwargs)

